###############################################
### RFS algorithm: rank and then sequentially select/filter
###############################################

#Definition of superiority/non-redundancy
## For univariate comparison between two arms (A, B) with respect to each assay j, for some eta>=0
#i) if mu_Aj-mu_Bj>eta for some j and mu_Bj-mu_Aj>eta for some j, then A & B are non-redundant wrt their immune profiles
#ii) if mu_Aj-mu_Bj>eta for some j, and mu_Bj-mu_Aj<=eta for every j, then A is superior to B wrt to its immune profile
#iii) if mu_Bj-mu_Aj>eta for some j, and mu_Aj-mu_Bj<=eta for every j, then B is superior to A wrt its immune profile
#iii) if |mu_Aj-mu_Bj|<=eta for every j, then A and B are equivalent wrt their immune profiles

# We conduct two-sided test comparing mean of each assay between each pair of regimens.
# Use Holm-Bonferroni procedure to control for PW-FNRE at a desired significance level.

#############################
## Input:
## all.data.list: input dataset for down-selection
## rank.method: method for creating summary score to rank regimens: "avg-score" for weighted average score, "avg-rank" for weighted average rank
## nmax: number of maximum possible number of regimens allowed to be selected
## wt.rank: the vector of weights given to each inidividual response variable X
## inferior.alapha: the PW-FNRE level to be controlled if 'holm' or 'holmt' is used in method.adjust, type-I error/2 if 'none' or 'nonet'
##     is used in method.adjust
## arm.name: name given to each regimen
## method.adjust: method to adjust for multiple comparison, including
## ---"holm" for Holm's method when Wald test is used for comparison;
## ---"none" for no adjustment when Wald test is used for comparison;
## ---"holmt" for Holm's method when t-test is used for comparison;
## ---"nonet" for no adjustment when t-test is used for comparison
## eta: the putative clinical difference (default to 0). That is, for comparison of an assay j between regimen A and B, define
## --- A superior to B if mu_Aj-mu_Bj>eta
## --- A inferior to B if mu_Bj-mu_Aj>eta
## --- A equivalent to B is |mu_Aj-mu_Bj|<=eta
## verbose=T

## Output
## arm.sorted: order of the regimens based on the univariate summary score
## res.selection: a vector of binary indicators about whether or not a regimen is selected (0 for not-selected, 1 for selected)
############################################################################################################################

seq.selection.fun3.eta <- function(all.data.list,
                               rank.method="avg-score", nmax=3, wt.rank=NULL,
                               inferior.alpha,
                               verbose=F, arm.name=NULL,method.adjust='holm',eta=0) {
  convergence=0

  # Data measurements
  Narm <- length(all.data.list)
  Nclass <- dim(all.data.list[[1]])[2]
  N <- dim(all.data.list[[1]])[1]
  if (is.null(arm.name)) arm.name <- as.character(1:Narm)

  if(Narm==1){
    res.selection <- 1
    names(res.selection) <- paste("Arm",arm.name,sep="")
    if (verbose) cat("Regimens selected: Arm ",arm.name,"\n")
    return(list(arm.sorted=as.numeric(arm.name), res.selection=res.selection))
  }

  # sorted arms
  if (is.null(wt.rank)) wt.rank <- rep(1,Nclass)
  arm.sorted <- rank.fun(all.data.list, rank.method, wt.rank)
  if(verbose) cat("Sorted arms: Arm ",arm.name[arm.sorted],"\n")

  # Prepare for univariate comparison

  # Mean/variance of scores
  summ.by.class.list <- lapply(all.data.list,function(x) {
    m <- apply(x,2,mean,na.rm=T)
    v <- apply(x,2,var,na.rm=T)
    n <- apply(!is.na(x),2,sum)  ### allow for missingness
    cbind(m=m,var=v,n=n)
  })
  summ.by.class <- do.call(rbind,summ.by.class.list)
  summ.by.class <- data.frame(Arm=rep(arm.name,each=Nclass),
                              variable=rep(paste("Class",1:Nclass,sep=""),Narm),
                              summ.by.class)
  # Weighted Score
  wt.by.arm.list <- lapply(all.data.list,function(x) {
    m <- apply(x,1,mean)
    c(m=mean(m),var=var(m))
  })
  wt.by.arm <- do.call(rbind,wt.by.arm.list)
  wt.by.arm <- data.frame(Arm=arm.name,wt.by.arm)
  wt.by.arm$variable <- "Summary Score"

  dat_all <- summ.by.class

  ###########
  dat.var.list <- lapply(all.data.list,function(x) {
    S=var(x)
    m <- apply(x,2,mean,na.rm=T)
    v <- apply(x,2,var,na.rm=T)
    n <- sum(apply(!is.na(x),1,prod))  ### allow for missingness
    list(S=S,m=m,v=v,n=n)
  })

  dat.var.Arm=arm.name


  # Return the arm with higher score for each immune class, add an eta
  uniclass.compare.fun <- function(dat_arm,arm.name,B,A,alpha,N,eta) {
    # Mean/SE/CI for difference in mean scores between two arms
    dat_m <- dcast(dat_arm,variable~Arm,value.var="m")
    # Assume "Higher response == Better"
    # B - A > eta indicates "B arm is better than A arm"
    dat_m$mDiff <- dat_m[,arm.name[B]]-dat_m[,arm.name[A]]
    dat_var <- dcast(dat_arm,variable~Arm,value.var="var")
    dat_n <- dcast(dat_arm,variable~Arm,value.var="n")

    dat_var$se <- sqrt((dat_var[,arm.name[B]]/dat_n[,arm.name[B]]+dat_var[,arm.name[A]]/dat_n[,arm.name[A]]))

    dat_m_ci <- merge(dat_m,dat_var,by="variable")
    dat_m_ci$lb <- dat_m_ci$mDiff-qnorm(1-alpha/2)*dat_m_ci$se - eta #(modified 05/08/2016)
    dat_m_ci$ub <- dat_m_ci$mDiff+qnorm(1-alpha/2)*dat_m_ci$se + eta #(modified 05/08/2016)
    dat_m_ci <- subset(dat_m_ci,select=c(variable,mDiff,lb,ub))
    dat_m_ci$sig <- ifelse(dat_m_ci$lb>0, "B", ifelse(dat_m_ci$ub<0, "A", "E"))
    dat_m_ci <- dat_m_ci[sapply(1:Nclass,function(j) which(dat_m_ci$variable==paste("Class",j,sep=""))),]
    dat_m_ci$sig
  }

  # Return the arm with higher score for each immune class, allow for missing data, add eta
  uniclass.compare.holm.fun <- function(dat_arm,arm.name,B,A,alpha,N,method,eta) {
    # Mean/SE/CI for difference in mean scores between two arms
    dat_m <- dcast(dat_arm,variable~Arm,value.var="m")
    # Assume "Higher response == Better"
    # B - A > eta indicates "B arm is better than A arm"
    dat_m$mDiff <- dat_m[,arm.name[B]]-dat_m[,arm.name[A]]
    dat_var <- dcast(dat_arm,variable~Arm,value.var="var")
    dat_n<- dcast(dat_arm,variable~Arm,value.var="n")
    dat_var$se <- sqrt((dat_var[,arm.name[B]]/dat_n[,arm.name[B]]+dat_var[,arm.name[A]]/dat_n[,arm.name[A]]))

    dat_m_ci <- merge(dat_m,dat_var,by="variable")


    # Holm-bonferroni adjustment

    dat_m_ci$pv.A<-1-pnorm((dat_m_ci$mDiff-eta)/dat_m_ci$se)  ## modified 05/08/2016
    dat_m_ci$pvh.A<-p.adjust(dat_m_ci$pv.A, method = method)

    dat_m_ci$pv.B<-pnorm((dat_m_ci$mDiff+eta)/dat_m_ci$se)    ## modified 05/08/2016
    dat_m_ci$pvh.B<-p.adjust(dat_m_ci$pv.B, method = method)

   dat_m_ci$sig<-ifelse(dat_m_ci$pvh.A<=alpha, "B", ifelse(dat_m_ci$pvh.B<=alpha, "A", "E"))  ## modified 05/08/2016

    dat_m_ci <- dat_m_ci[sapply(1:Nclass,function(j) which(dat_m_ci$variable==paste("Class",j,sep=""))),]
    dat_m_ci$sig
 }

 ### modified 05/08/2016
  uniclass.compare.holmt.fun <- function(dat_arm,arm.name,B,A,alpha,N,method,eta) {
    # Mean/SE/CI for difference in mean scores between two arms
    dat_m <- dcast(dat_arm,variable~Arm,value.var="m")
    # Assume "Higher response == Better"
    # B - A > eta indicates "B arm is better than A arm"
    dat_m$mDiff <- dat_m[,arm.name[B]]-dat_m[,arm.name[A]]
    dat_var <- dcast(dat_arm,variable~Arm,value.var="var")
    dat_n<- dcast(dat_arm,variable~Arm,value.var="n")
    dat_var$se <- sqrt((dat_var[,arm.name[B]]*(dat_n[,arm.name[B]]-1)+
    dat_var[,arm.name[A]]*(dat_n[,arm.name[A]]-1))/(dat_n[,arm.name[B]]+dat_n[,arm.name[A]]-2))*
    sqrt(1/dat_n[,arm.name[B]]+1/dat_n[,arm.name[A]])

    dat_var$nA<-dat_n[,arm.name[A]]
    dat_var$nB<-dat_n[,arm.name[B]]
    dat_m_ci <- merge(dat_m,dat_var,by="variable")

    # Holm-bonferroni adjustment


    dat_m_ci$pv.A<-1-pt((dat_m_ci$mDiff-eta)/dat_m_ci$se,dat_m_ci$nA+dat_m_ci$nB-2)  ## modified 05/08/2016
    dat_m_ci$pvh.A<-p.adjust(dat_m_ci$pv.A, method = method)

    dat_m_ci$pv.B<-pt((dat_m_ci$mDiff+eta)/dat_m_ci$se,dat_m_ci$nA+dat_m_ci$nB-2)  ## modified 05/08/2016
    dat_m_ci$pvh.B<-p.adjust(dat_m_ci$pv.B, method = method)

     dat_m_ci$sig<-ifelse(dat_m_ci$pvh.A<=alpha, "B", ifelse(dat_m_ci$pvh.B<=alpha, "A", "E"))  ## modified 05/08/2016

    dat_m_ci <- dat_m_ci[sapply(1:Nclass,function(j) which(dat_m_ci$variable==paste("Class",j,sep=""))),]
    dat_m_ci$sig
 }



  # Sequential selection of regimens

  # 1. start with the top rank arm
  selected.arm <- arm.sorted[1]
  arm.to.select <- arm.sorted[-1]

  if(verbose) cat("Selected:", selected.arm, "\n")
  if(verbose) cat("To be Selected:", arm.to.select, "\n")

  while( length(arm.to.select)>0 & length(selected.arm)<nmax ) {

    B <- arm.to.select[1]
    DECISION <- rep(NA,length(selected.arm))

    for (j in 1:length(selected.arm)) {
      # A always has higher rank than B
      A <- selected.arm[j]

        dat_arm <- subset(dat_all,Arm==arm.name[B]|Arm==arm.name[A])

        # 2.a.1 test for superior, equivalent, or inferior (sig+ indicates = B - A > eta)


        if (method.adjust=='holmt'){

           high<-uniclass.compare.holmt.fun(dat_arm,arm.name,B,A,inferior.alpha,N,'holm',eta) ## modified 05/08/2016

        }  else if (method.adjust=='nonet'){

          high<-uniclass.compare.holmt.fun(dat_arm,arm.name,B,A,inferior.alpha,N,'none',eta) ## modified 05/08/2016

        }  else {
        high <- uniclass.compare.holm.fun(dat_arm,arm.name,B,A,inferior.alpha,N,method.adjust,eta) ## modified 05/08/2016
        }


        if(all(high == "E")) DECISION[j] <- "A"
        if(any(high == "A") & any(high == "B")) DECISION[j] <- "Both"
        if(any(high == "A") & !any(high == "B")) DECISION[j] <- "A"
        if(!any(high == "A") & any(high == "B")) DECISION[j] <- "B"

    }

    if(convergence==1) break

    if(all(DECISION=="Both")){
      selected.arm <- c(selected.arm,B)
      arm.to.select <- arm.to.select[-1]
    }
    if(all(DECISION=="B")){
      selected.arm <- B
      arm.to.select <- arm.to.select[-1]
    }
    if(any(DECISION=="A")){
      selected.arm <- selected.arm
      arm.to.select <- arm.to.select[-1]
    }
    if(all(DECISION!="A") & any(DECISION=="B") & any(DECISION=="Both")){
      selected.arm <- c(selected.arm[DECISION=="Both"],B)
      arm.to.select <- arm.to.select[-1]
    }

    if(verbose) cat("Selected:", selected.arm, "\n")
    if(verbose) cat("To be Selected:", arm.to.select, "\n")
  }

  res.selection <- 1*arm.name %in% arm.name[selected.arm]
  arm.sorted <- as.numeric(arm.name[arm.sorted])

  if (convergence==1)
    arm.sorted <- res.selection <- rep(NA,Narm)

  names(res.selection) <- paste("Arm",arm.name,sep="")
  return(list(arm.sorted=arm.sorted, res.selection=res.selection))
}






###############################################
### Simulation data
###############################################

### Function to generate simulation data

# N = sample size per arm
# Narm = # of arms
# S = covariance matrix (same across arms)
# M = mean vector (each row represents an arm; each column represents a class)

### return a data list of length Narm

### simulate from multivariate normal
simudata.fun <- function(N,Narm,S,M) {
  all.data <- list(NULL)
  for (k in 1:Narm) {
    all.data[[k]] <- mvrnorm(n=N,mu=M[k,],Sigma=S)
  }
  all.data
}




try.error<-function(expr, silent=TRUE) {

  # make it to error
  op <- options("warn")
  on.exit(options(op))
  options(warn = -1)

  # catch using the try() function
  try(expr, silent=TRUE)
}



###############################################
### Ranking
###############################################

# all.data.list = list. output of simudata.fun()
# rank.method = method to do ranking. Detail:
#   "avg-rank":         average rank across classes
#   "avg-score":        average raw scores
#   "avg-rank-score1":  average transformed raw scores (transformed to rank)
#   "avg-rank-score2":  average transformed raw scores (transformed to score 1-4 based on N(0,1) quartiles)
# wt.rank = a vector of class specific weights if rank.method == "avg-rank" or "ave-score"(default 1 for every class)

# return a vector of sorted arms based on their ranks in ascending order (low rank indicates high score)

rank.fun <- function(all.data.list, rank.method=c("avg-rank","avg-score","avg-trans-score1","avg-trans-score2"),
                     wt.rank=NULL) {

  # Data measurements
  Narm <- length(all.data.list)
  Nclass <- dim(all.data.list[[1]])[2]
  N <- dim(all.data.list[[1]])[1]

  if (rank.method == "avg-rank") {
    # Class specific weights
    if (is.null(wt.rank)) wt.rank <- rep(1,Nclass)

    score.list <- all.data.list

    # Mean score within each class by arm
    meanlist <- lapply(score.list,function(x) apply(x,2,mean,na.rm=T))
    meanmat <- matrix(unlist(meanlist),nrow=Narm,ncol=Nclass,byrow=T)

    # Rank by mean score
    rankmat <- apply(meanmat,2,function(u) rank(-u,ties.method="random"))
    #rankmat <- apply(meanmat,2,function(u) rank(-u,ties.method="first"))

    avgrank <- apply(rankmat,1,weighted.mean,wt.rank)
    final.rank <- rank(avgrank,ties.method="random")
    #final.rank <- rank(avgrank,ties.method="first")

  }

  if (rank.method == "avg-score") {
    score.list <- all.data.list

    # Mean score within each class by arm
    meanlist <- lapply(score.list,function(x) apply(x,2,mean,na.rm=T))
    meanmat <- matrix(unlist(meanlist),nrow=Narm,ncol=Nclass,byrow=T)

    # Rank by mean score
    avgmean <- apply(meanmat,1,weighted.mean,wt.rank)
    final.rank <- rank(-avgmean,ties.method="random")
  }

  if (rank.method == "avg-trans-score1") {
    # Data in matrix (long) form
    all.data.matrix <- do.call(rbind, all.data.list)

    # Rank-transformation
    trans.rankmat <- apply(all.data.matrix,2,function(u) rank(u,ties.method="random"))
    trans.ranklist <- list()
    for (i in 1:Narm) {
      trans.ranklist[[i]] <- trans.rankmat[(1+N*(i-1)):(N*i),]
    }
    score.list <- trans.ranklist

    # Mean trans-score within each class by arm
    meanlist <- lapply(score.list,function(x) apply(x,2,mean))
    meanmat <- matrix(unlist(meanlist),nrow=Narm,ncol=Nclass,byrow=T)

    # Rank by mean transfored score
    avgmean <- apply(meanmat,1,mean)
    final.rank <- rank(-avgmean,ties.method="random")
  }

  if (rank.method == "avg-trans-score2") {

    standNquar <- qnorm(c(0.25,0.5,0.75))
    transx <- function(x) ifelse(x<=standNquar[1],1,ifelse(x<=standNquar[2],2,ifelse(x<=standNquar[3],3,4)))

    # Rank-transformation
    trans.ranklist <- lapply(all.data.list,transx)
    score.list <- trans.ranklist

    # Mean trans-score within each class by arm
    meanlist <- lapply(score.list,function(x) apply(x,2,mean))
    meanmat <- matrix(unlist(meanlist),nrow=Narm,ncol=Nclass,byrow=T)

    # Rank by mean score
    avgmean <- apply(meanmat,1,mean)
    final.rank <- rank(-avgmean,ties.method="random")
  }

  # Sort the arms by final.rank (low rank indicates high score)
  arm.sorted <- c(1:Narm)[order(final.rank)]
  arm.sorted
}




#########################################################
## Function to compute summary score for each regimen
###
### input
### M: mean struture, row for regimen, col for immune response
### wt.rank: vector of weights given to immune response

### output
### rank.AR: weighted average rank
### rank.AS: weighted average score
### dmat: distance matrix
#########################################################

######## the function output
summaryFun <- function(M,wt.rank) {
  rank.class <- apply(-M,2,rank)
  ave.rank <- apply(rank.class,1,weighted.mean,wt.rank)
  mean.score <- apply(M,1,weighted.mean,wt.rank)

  oo.AR <- order(ave.rank)
  oo.AS <- order(-mean.score)

  Nclass <- ncol(M)
  mscore <- M
  mscore.wt <- sweep(mscore,2,sqrt(wt.rank),"*")
  dmat <- as.matrix(dist(mscore.wt, method = "euclidean", diag = TRUE,upper=TRUE))

  return(list(rank.AR=cbind(oo.AR,ave.rank[oo.AR]),rank.AS=cbind(oo.AS,mean.score[oo.AS]),dmat=dmat))
}
